@@ -7,13 +7,12 @@ import sys
 from itertools import chain
 
 import py
-from pkg_resources import to_filename
 
 import tox
 from tox import reporter
 from tox.action import Action
-from tox.config.parallel import ENV_VAR_KEY as PARALLEL_ENV_VAR_KEY
-from tox.constants import PARALLEL_RESULT_JSON_PREFIX, PARALLEL_RESULT_JSON_SUFFIX
+from tox.config.parallel import ENV_VAR_KEY_PRIVATE as PARALLEL_ENV_VAR_KEY_PRIVATE
+from tox.constants import INFO, PARALLEL_RESULT_JSON_PREFIX, PARALLEL_RESULT_JSON_SUFFIX
 from tox.package.local import resolve_package
 from tox.util.lock import get_unique_file
 from tox.util.path import ensure_empty_dir
@@ -24,7 +23,7 @@ from .config import DepConfig
 class CreationConfig:
     def __init__(
         self,
-        base_resolved_python_md5,
+        base_resolved_python_sha256,
         base_resolved_python_path,
         tox_version,
         sitepackages,
@@ -32,7 +31,7 @@ class CreationConfig:
         deps,
         alwayscopy,
     ):
-        self.base_resolved_python_md5 = base_resolved_python_md5
+        self.base_resolved_python_sha256 = base_resolved_python_sha256
         self.base_resolved_python_path = base_resolved_python_path
         self.tox_version = tox_version
         self.sitepackages = sitepackages
@@ -42,7 +41,7 @@ class CreationConfig:
 
     def writeconfig(self, path):
         lines = [
-            "{} {}".format(self.base_resolved_python_md5, self.base_resolved_python_path),
+            "{} {}".format(self.base_resolved_python_sha256, self.base_resolved_python_path),
             "{} {:d} {:d} {:d}".format(
                 self.tox_version, self.sitepackages, self.usedevelop, self.alwayscopy
             ),
@@ -65,11 +64,11 @@ class CreationConfig:
             alwayscopy = bool(int(alwayscopy))
             deps = []
             for line in lines:
-                base_resolved_python_md5, depstring = line.split(None, 1)
-                deps.append((base_resolved_python_md5, depstring))
-            base_resolved_python_md5, base_resolved_python_path = base_resolved_python_info
+                base_resolved_python_sha256, depstring = line.split(None, 1)
+                deps.append((base_resolved_python_sha256, depstring))
+            base_resolved_python_sha256, base_resolved_python_path = base_resolved_python_info
             return CreationConfig(
-                base_resolved_python_md5,
+                base_resolved_python_sha256,
                 base_resolved_python_path,
                 tox_version,
                 sitepackages,
@@ -82,7 +81,7 @@ class CreationConfig:
 
     def matches_with_reason(self, other, deps_matches_subset=False):
         for attr in (
-            "base_resolved_python_md5",
+            "base_resolved_python_sha256",
             "base_resolved_python_path",
             "tox_version",
             "sitepackages",
@@ -131,6 +130,8 @@ class VirtualEnv(object):
             command_log,
             self.popen,
             self.envconfig.envpython,
+            self.envconfig.interrupt_timeout,
+            self.envconfig.terminate_timeout,
         )
 
     def get_result_json_path(self):
@@ -267,11 +268,11 @@ class VirtualEnv(object):
         alwayscopy = self.envconfig.alwayscopy
         deps = []
         for dep in self.get_resolved_dependencies():
-            dep_name_md5 = getdigest(dep.name)
-            deps.append((dep_name_md5, dep.name))
-        base_resolved_python_md5 = getdigest(base_resolved_python_path)
+            dep_name_sha256 = getdigest(dep.name)
+            deps.append((dep_name_sha256, dep.name))
+        base_resolved_python_sha256 = getdigest(base_resolved_python_path)
         return CreationConfig(
-            base_resolved_python_md5,
+            base_resolved_python_sha256,
             base_resolved_python_path,
             version,
             sitepackages,
@@ -324,7 +325,7 @@ class VirtualEnv(object):
             sys_path = json.loads(out)
         except ValueError:
             sys_path = []
-        egg_info_fname = ".".join((to_filename(name), "egg-info"))
+        egg_info_fname = ".".join((name.replace("-", "_"), "egg-info"))
         for d in reversed(sys_path):
             egg_info = py.path.local(d).join(egg_info_fname)
             if egg_info.check():
@@ -576,11 +577,11 @@ class VirtualEnv(object):
         )
 
     def setupenv(self):
-        if self.envconfig.missing_subs:
+        if self.envconfig._missing_subs:
             self.status = (
                 "unresolvable substitution(s): {}. "
                 "Environment variables are missing or defined recursively.".format(
-                    ",".join(["'{}'".format(m) for m in self.envconfig.missing_subs])
+                    ",".join(["'{}'".format(m) for m in self.envconfig._missing_subs])
                 )
             )
             return
@@ -630,7 +631,7 @@ def getdigest(path):
     path = py.path.local(path)
     if not path.check(file=1):
         return "0" * 32
-    return path.computehash()
+    return path.computehash("sha256")
 
 
 def prepend_shebang_interpreter(args):
@@ -694,7 +695,36 @@ def tox_testenv_create(venv, action):
 
 
 def cleanup_for_venv(venv):
-    within_parallel = PARALLEL_ENV_VAR_KEY in os.environ
+    within_parallel = PARALLEL_ENV_VAR_KEY_PRIVATE in os.environ
+    # if the directory exists and it doesn't look like a virtualenv, produce
+    # an error
+    if venv.path.exists():
+        dir_items = set(os.listdir(str(venv.path))) - {".lock", "log"}
+        dir_items = {p for p in dir_items if not p.startswith(".tox-") or p == ".tox-config1"}
+    else:
+        dir_items = set()
+
+    if not (
+        # doesn't exist => OK
+        not venv.path.exists()
+        # does exist, but it's empty => OK
+        or not dir_items
+        # tox has marked this as an environment it has created in the past
+        or ".tox-config1" in dir_items
+        # it exists and we're on windows with Lib and Scripts => OK
+        or (INFO.IS_WIN and dir_items > {"Scripts", "Lib"})
+        # non-windows, with lib and bin => OK
+        or dir_items > {"bin", "lib"}
+        # pypy has a different lib folder => OK
+        or dir_items > {"bin", "lib_pypy"}
+    ):
+        venv.status = "error"
+        reporter.error(
+            "cowardly refusing to delete `envdir` (it does not look like a virtualenv): "
+            "{}".format(venv.path)
+        )
+        raise SystemExit(2)
+
     if within_parallel:
         if venv.path.exists():
             # do not delete the log folder as that's used by parent
